@@ -24,14 +24,14 @@ var Fetch = {
   openDatabase: function(dbname, dbversion, onsuccess, onerror) {
     try {
 #if FETCH_DEBUG
-      console.log('fetch: indexedDB.open(dbname="' + dbname + '", dbversion="' + dbversion + '");');
+      dbg('fetch: indexedDB.open(dbname="' + dbname + '", dbversion="' + dbversion + '");');
 #endif
       var openRequest = indexedDB.open(dbname, dbversion);
     } catch (e) { return onerror(e); }
 
     openRequest.onupgradeneeded = (event) => {
 #if FETCH_DEBUG
-      console.log('fetch: IndexedDB upgrade needed. Clearing database.');
+      dbg('fetch: IndexedDB upgrade needed. Clearing database.');
 #endif
       var db = /** @type {IDBDatabase} */ (event.target.result);
       if (db.objectStoreNames.contains('FILES')) {
@@ -50,7 +50,7 @@ var Fetch = {
 #if FETCH_SUPPORT_INDEXEDDB
     var onsuccess = (db) => {
 #if FETCH_DEBUG
-      console.log('fetch: IndexedDB successfully opened.');
+      dbg('fetch: IndexedDB successfully opened.');
 #endif
       Fetch.dbInstance = db;
 
@@ -60,7 +60,7 @@ var Fetch = {
     };
     var onerror = () => {
 #if FETCH_DEBUG
-      console.error('fetch: IndexedDB open failed.');
+      dbg('fetch: IndexedDB open failed.');
 #endif
       Fetch.dbInstance = false;
 
@@ -81,7 +81,7 @@ var Fetch = {
 function fetchDeleteCachedData(db, fetch, onsuccess, onerror) {
   if (!db) {
 #if FETCH_DEBUG
-    console.error('fetch: IndexedDB not available!');
+    dbg('fetch: IndexedDB not available!');
 #endif
     onerror(fetch, 0, 'IndexedDB not available!');
     return;
@@ -99,7 +99,7 @@ function fetchDeleteCachedData(db, fetch, onsuccess, onerror) {
     request.onsuccess = (event) => {
       var value = event.target.result;
 #if FETCH_DEBUG
-      console.log('fetch: Deleted file ' + pathStr + ' from IndexedDB');
+      dbg('fetch: Deleted file ' + pathStr + ' from IndexedDB');
 #endif
       HEAPU32[fetch + {{{ C_STRUCTS.emscripten_fetch_t.data }}} >> 2] = 0;
       Fetch.setu64(fetch + {{{ C_STRUCTS.emscripten_fetch_t.numBytes }}}, 0);
@@ -112,7 +112,7 @@ function fetchDeleteCachedData(db, fetch, onsuccess, onerror) {
     };
     request.onerror = (error) => {
 #if FETCH_DEBUG
-      console.error('fetch: Failed to delete file ' + pathStr + ' from IndexedDB! error: ' + error);
+      dbg('fetch: Failed to delete file ' + pathStr + ' from IndexedDB! error: ' + error);
 #endif
       HEAPU16[fetch + {{{ C_STRUCTS.emscripten_fetch_t.readyState }}} >> 1] = 4; // Mimic XHR readyState 4 === 'DONE: The operation is complete'
       HEAPU16[fetch + {{{ C_STRUCTS.emscripten_fetch_t.status }}} >> 1] = 404; // Mimic XHR HTTP status code 404 "Not Found"
@@ -121,7 +121,7 @@ function fetchDeleteCachedData(db, fetch, onsuccess, onerror) {
     };
   } catch(e) {
 #if FETCH_DEBUG
-    console.error('fetch: Failed to load file ' + pathStr + ' from IndexedDB! Got exception ' + e);
+    dbg('fetch: Failed to load file ' + pathStr + ' from IndexedDB! Got exception ' + e);
 #endif
     onerror(fetch, 0, e);
   }
@@ -130,7 +130,7 @@ function fetchDeleteCachedData(db, fetch, onsuccess, onerror) {
 function fetchLoadCachedData(db, fetch, onsuccess, onerror) {
   if (!db) {
 #if FETCH_DEBUG
-    console.error('fetch: IndexedDB not available!');
+    dbg('fetch: IndexedDB not available!');
 #endif
     onerror(fetch, 0, 'IndexedDB not available!');
     return;
@@ -150,7 +150,7 @@ function fetchLoadCachedData(db, fetch, onsuccess, onerror) {
         var value = event.target.result;
         var len = value.byteLength || value.length;
 #if FETCH_DEBUG
-        console.log('fetch: Loaded file ' + pathStr + ' from IndexedDB, length: ' + len);
+        dbg('fetch: Loaded file ' + pathStr + ' from IndexedDB, length: ' + len);
 #endif
         // The data pointer malloc()ed here has the same lifetime as the emscripten_fetch_t structure itself has, and is
         // freed when emscripten_fetch_close() is called.
@@ -167,7 +167,7 @@ function fetchLoadCachedData(db, fetch, onsuccess, onerror) {
       } else {
         // Succeeded to load, but the load came back with the value of undefined, treat that as an error since we never store undefined in db.
 #if FETCH_DEBUG
-        console.error('fetch: File ' + pathStr + ' not found in IndexedDB');
+        dbg('fetch: File ' + pathStr + ' not found in IndexedDB');
 #endif
         HEAPU16[fetch + {{{ C_STRUCTS.emscripten_fetch_t.readyState }}} >> 1] = 4; // Mimic XHR readyState 4 === 'DONE: The operation is complete'
         HEAPU16[fetch + {{{ C_STRUCTS.emscripten_fetch_t.status }}} >> 1] = 404; // Mimic XHR HTTP status code 404 "Not Found"
@@ -177,7 +177,7 @@ function fetchLoadCachedData(db, fetch, onsuccess, onerror) {
     };
     getRequest.onerror = (error) => {
 #if FETCH_DEBUG
-      console.error('fetch: Failed to load file ' + pathStr + ' from IndexedDB!');
+      dbg('fetch: Failed to load file ' + pathStr + ' from IndexedDB!');
 #endif
       HEAPU16[fetch + {{{ C_STRUCTS.emscripten_fetch_t.readyState }}} >> 1] = 4; // Mimic XHR readyState 4 === 'DONE: The operation is complete'
       HEAPU16[fetch + {{{ C_STRUCTS.emscripten_fetch_t.status }}} >> 1] = 404; // Mimic XHR HTTP status code 404 "Not Found"
@@ -186,7 +186,7 @@ function fetchLoadCachedData(db, fetch, onsuccess, onerror) {
     };
   } catch(e) {
 #if FETCH_DEBUG
-    console.error('fetch: Failed to load file ' + pathStr + ' from IndexedDB! Got exception ' + e);
+    dbg('fetch: Failed to load file ' + pathStr + ' from IndexedDB! Got exception ' + e);
 #endif
     onerror(fetch, 0, e);
   }
@@ -195,7 +195,7 @@ function fetchLoadCachedData(db, fetch, onsuccess, onerror) {
 function fetchCacheData(/** @type {IDBDatabase} */ db, fetch, data, onsuccess, onerror) {
   if (!db) {
 #if FETCH_DEBUG
-    console.error('fetch: IndexedDB not available!');
+    dbg('fetch: IndexedDB not available!');
 #endif
     onerror(fetch, 0, 'IndexedDB not available!');
     return;
@@ -212,7 +212,7 @@ function fetchCacheData(/** @type {IDBDatabase} */ db, fetch, data, onsuccess, o
     var putRequest = packages.put(data, destinationPathStr);
     putRequest.onsuccess = (event) => {
 #if FETCH_DEBUG
-      console.log('fetch: Stored file "' + destinationPathStr + '" to IndexedDB cache.');
+      dbg('fetch: Stored file "' + destinationPathStr + '" to IndexedDB cache.');
 #endif
       HEAPU16[fetch + {{{ C_STRUCTS.emscripten_fetch_t.readyState }}} >> 1] = 4; // Mimic XHR readyState 4 === 'DONE: The operation is complete'
       HEAPU16[fetch + {{{ C_STRUCTS.emscripten_fetch_t.status }}} >> 1] = 200; // Mimic XHR HTTP status code 200 "OK"
@@ -221,7 +221,7 @@ function fetchCacheData(/** @type {IDBDatabase} */ db, fetch, data, onsuccess, o
     };
     putRequest.onerror = (error) => {
 #if FETCH_DEBUG
-      console.error('fetch: Failed to store file "' + destinationPathStr + '" to IndexedDB cache!');
+      dbg('fetch: Failed to store file "' + destinationPathStr + '" to IndexedDB cache!');
 #endif
       // Most likely we got an error if IndexedDB is unwilling to store any more data for this page.
       // TODO: Can we identify and break down different IndexedDB-provided errors and convert those
@@ -233,7 +233,7 @@ function fetchCacheData(/** @type {IDBDatabase} */ db, fetch, data, onsuccess, o
     };
   } catch(e) {
 #if FETCH_DEBUG
-      console.error('fetch: Failed to store file "' + destinationPathStr + '" to IndexedDB cache! Exception: ' + e);
+      dbg('fetch: Failed to store file "' + destinationPathStr + '" to IndexedDB cache! Exception: ' + e);
 #endif
     onerror(fetch, 0, e);
   }
@@ -244,7 +244,7 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
   var url = HEAPU32[fetch + {{{ C_STRUCTS.emscripten_fetch_t.url }}} >> 2];
   if (!url) {
 #if FETCH_DEBUG
-    console.error('fetch: XHR failed, no URL specified!');
+    dbg('fetch: XHR failed, no URL specified!');
 #endif
     onerror(fetch, 0, 'no url specified!');
     return;
@@ -282,8 +282,8 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
   var xhr = new XMLHttpRequest();
   xhr.withCredentials = withCredentials;
 #if FETCH_DEBUG
-  console.log('fetch: xhr.timeout: ' + xhr.timeout + ', xhr.withCredentials: ' + xhr.withCredentials);
-  console.log('fetch: xhr.open(requestMethod="' + requestMethod + '", url: "' + url_ +'", userName: ' + userNameStr + ', password: ' + passwordStr + ');');
+  dbg('fetch: xhr.timeout: ' + xhr.timeout + ', xhr.withCredentials: ' + xhr.withCredentials);
+  dbg('fetch: xhr.open(requestMethod="' + requestMethod + '", url: "' + url_ +'", userName: ' + userNameStr + ', password: ' + passwordStr + ');');
 #endif
   xhr.open(requestMethod, url_, !fetchAttrSynchronous, userNameStr, passwordStr);
   if (!fetchAttrSynchronous) xhr.timeout = timeoutMsecs; // XHR timeout field is only accessible in async XHRs, and must be set after .open() but before .send().
@@ -296,7 +296,7 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
   if (overriddenMimeType) {
     var overriddenMimeTypeStr = UTF8ToString(overriddenMimeType);
 #if FETCH_DEBUG
-    console.log('fetch: xhr.overrideMimeType("' + overriddenMimeTypeStr + '");');
+    dbg('fetch: xhr.overrideMimeType("' + overriddenMimeTypeStr + '");');
 #endif
     xhr.overrideMimeType(overriddenMimeTypeStr);
   }
@@ -310,7 +310,7 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
       var keyStr = UTF8ToString(key);
       var valueStr = UTF8ToString(value);
 #if FETCH_DEBUG
-      console.log('fetch: xhr.setRequestHeader("' + keyStr + '", "' + valueStr + '");');
+      dbg('fetch: xhr.setRequestHeader("' + keyStr + '", "' + valueStr + '");');
 #endif
       xhr.setRequestHeader(keyStr, valueStr);
     }
@@ -332,7 +332,7 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
     }
     if (ptrLen > 0) {
 #if FETCH_DEBUG
-      console.log('fetch: allocating ' + ptrLen + ' bytes in Emscripten heap for xhr data');
+      dbg('fetch: allocating ' + ptrLen + ' bytes in Emscripten heap for xhr data');
 #endif
       // The data pointer malloc()ed here has the same lifetime as the emscripten_fetch_t structure itself has, and is
       // freed when emscripten_fetch_close() is called.
@@ -362,12 +362,12 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
     saveResponseAndStatus();
     if (xhr.status >= 200 && xhr.status < 300) {
 #if FETCH_DEBUG
-      console.log('fetch: xhr of URL "' + xhr.url_ + '" / responseURL "' + xhr.responseURL + '" succeeded with status ' + xhr.status);
+      dbg('fetch: xhr of URL "' + xhr.url_ + '" / responseURL "' + xhr.responseURL + '" succeeded with status ' + xhr.status);
 #endif
       if (onsuccess) onsuccess(fetch, xhr, e);
     } else {
 #if FETCH_DEBUG
-      console.error('fetch: xhr of URL "' + xhr.url_ + '" / responseURL "' + xhr.responseURL + '" failed with status ' + xhr.status);
+      dbg('fetch: xhr of URL "' + xhr.url_ + '" / responseURL "' + xhr.responseURL + '" failed with status ' + xhr.status);
 #endif
       if (onerror) onerror(fetch, xhr, e);
     }
@@ -378,7 +378,7 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
       return;
     }
 #if FETCH_DEBUG
-    console.error('fetch: xhr of URL "' + xhr.url_ + '" / responseURL "' + xhr.responseURL + '" finished with error, readyState ' + xhr.readyState + ' and status ' + xhr.status);
+    dbg('fetch: xhr of URL "' + xhr.url_ + '" / responseURL "' + xhr.responseURL + '" finished with error, readyState ' + xhr.readyState + ' and status ' + xhr.status);
 #endif
     saveResponseAndStatus();
     if (onerror) onerror(fetch, xhr, e);
@@ -389,7 +389,7 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
       return;
     }
 #if FETCH_DEBUG
-    console.error('fetch: xhr of URL "' + xhr.url_ + '" / responseURL "' + xhr.responseURL + '" timed out, readyState ' + xhr.readyState + ' and status ' + xhr.status);
+    dbg('fetch: xhr of URL "' + xhr.url_ + '" / responseURL "' + xhr.responseURL + '" timed out, readyState ' + xhr.readyState + ' and status ' + xhr.status);
 #endif
     if (onerror) onerror(fetch, xhr, e);
   };
@@ -402,7 +402,7 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
     var ptr = 0;
     if (ptrLen > 0 && fetchAttrLoadToMemory && fetchAttrStreamData) {
 #if FETCH_DEBUG
-      console.log('fetch: allocating ' + ptrLen + ' bytes in Emscripten heap for xhr data');
+      dbg('fetch: allocating ' + ptrLen + ' bytes in Emscripten heap for xhr data');
 #endif
 #if ASSERTIONS
       assert(onprogress, 'When doing a streaming fetch, you should have an onprogress handler registered to receive the chunks!');
@@ -438,13 +438,13 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
     if (onreadystatechange) onreadystatechange(fetch, xhr, e);
   };
 #if FETCH_DEBUG
-  console.log('fetch: xhr.send(data=' + data + ')');
+  dbg('fetch: xhr.send(data=' + data + ')');
 #endif
   try {
     xhr.send(data);
   } catch(e) {
 #if FETCH_DEBUG
-    console.error('fetch: xhr failed with exception: ' + e);
+    dbg('fetch: xhr failed with exception: ' + e);
 #endif
     if (onerror) onerror(fetch, xhr, e);
   }
@@ -482,7 +482,7 @@ function startFetch(fetch, successcb, errorcb, progresscb, readystatechangecb) {
 
   var reportSuccess = (fetch, xhr, e) => {
 #if FETCH_DEBUG
-    console.log('fetch: operation success. e: ' + e);
+    dbg('fetch: operation success. e: ' + e);
 #endif
     {{{ runtimeKeepalivePop() }}}
     doCallback(() => {
@@ -500,7 +500,7 @@ function startFetch(fetch, successcb, errorcb, progresscb, readystatechangecb) {
 
   var reportError = (fetch, xhr, e) => {
 #if FETCH_DEBUG
-    console.error('fetch: operation failed: ' + e);
+    dbg('fetch: operation failed: ' + e);
 #endif
     {{{ runtimeKeepalivePop() }}}
     doCallback(() => {
@@ -511,7 +511,7 @@ function startFetch(fetch, successcb, errorcb, progresscb, readystatechangecb) {
 
   var reportReadyStateChange = (fetch, xhr, e) => {
 #if FETCH_DEBUG
-    console.log('fetch: ready state change. e: ' + e);
+    dbg('fetch: ready state change. e: ' + e);
 #endif
     doCallback(() => {
       if (onreadystatechange) {{{ makeDynCall('vp', 'onreadystatechange') }}}(fetch);
@@ -521,7 +521,7 @@ function startFetch(fetch, successcb, errorcb, progresscb, readystatechangecb) {
 
   var performUncachedXhr = (fetch, xhr, e) => {
 #if FETCH_DEBUG
-    console.error('fetch: starting (uncached) XHR: ' + e);
+    dbg('fetch: starting (uncached) XHR: ' + e);
 #endif
     fetchXHR(fetch, reportSuccess, reportError, reportProgress, reportReadyStateChange);
   };
@@ -529,11 +529,11 @@ function startFetch(fetch, successcb, errorcb, progresscb, readystatechangecb) {
 #if FETCH_SUPPORT_INDEXEDDB
   var cacheResultAndReportSuccess = (fetch, xhr, e) => {
 #if FETCH_DEBUG
-    console.log('fetch: operation success. Caching result.. e: ' + e);
+    dbg('fetch: operation success. Caching result.. e: ' + e);
 #endif
     var storeSuccess = (fetch, xhr, e) => {
 #if FETCH_DEBUG
-      console.log('fetch: IndexedDB store succeeded.');
+      dbg('fetch: IndexedDB store succeeded.');
 #endif
       {{{ runtimeKeepalivePop() }}}
       doCallback(() => {
@@ -543,7 +543,7 @@ function startFetch(fetch, successcb, errorcb, progresscb, readystatechangecb) {
     };
     var storeError = (fetch, xhr, e) => {
 #if FETCH_DEBUG
-      console.error('fetch: IndexedDB store failed.');
+      dbg('fetch: IndexedDB store failed.');
 #endif
       {{{ runtimeKeepalivePop() }}}
       doCallback(() => {
@@ -556,7 +556,7 @@ function startFetch(fetch, successcb, errorcb, progresscb, readystatechangecb) {
 
   var performCachedXhr = (fetch, xhr, e) => {
 #if FETCH_DEBUG
-    console.error('fetch: starting (cached) XHR: ' + e);
+    dbg('fetch: starting (cached) XHR: ' + e);
 #endif
     fetchXHR(fetch, cacheResultAndReportSuccess, reportError, reportProgress, reportReadyStateChange);
   };
@@ -573,7 +573,7 @@ function startFetch(fetch, successcb, errorcb, progresscb, readystatechangecb) {
     fetchXHR(fetch, fetchAttrPersistFile ? cacheResultAndReportSuccess : reportSuccess, reportError, reportProgress, reportReadyStateChange);
   } else {
 #if FETCH_DEBUG
-    console.error('fetch: Invalid combination of flags passed.');
+    dbg('fetch: Invalid combination of flags passed.');
 #endif
     return 0; // todo: free
   }
@@ -598,7 +598,7 @@ function fetchGetResponseHeaders(id, dst, dstSizeBytes) {
 //Delete the xhr JS object, allowing it to be garbage collected.
 function fetchFree(id) {
 #if FETCH_DEBUG
-    console.log("fetch: Deleting id:" + id + " of " + Fetch.xhrs);
+    dbg("fetch: Deleting id:" + id + " of " + Fetch.xhrs);
 #endif
     var xhr = Fetch.xhrs[id];
     if (xhr) {
