@@ -1,9 +1,10 @@
+/**
+ * @license
+ * Copyright 2014 The Emscripten Authors
+ * SPDX-License-Identifier: MIT
+ */
+
 /*
- * Copyright 2014 The Emscripten Authors.  All rights reserved.
- * Emscripten is available under two separate licenses, the MIT license and the
- * University of Illinois/NCSA Open Source License.  Both these licenses can be
- * found in the LICENSE file.
- *
  * EMSCRIPTEN GLEW 1.10.0 emulation
  *
  * What it does:
@@ -19,7 +20,7 @@
  */
 
 var LibraryGLEW = {
-  $GLEW__deps: ['glGetString'],
+  $GLEW__deps: ['glGetString', '$allocateUTF8'],
   $GLEW: {
     isLinaroFork: 1,
     extensions: null,
@@ -72,7 +73,7 @@ var LibraryGLEW = {
           string = "Unknown error";
           error = 8; // prevent array from growing more than this
         }
-        GLEW.error[error] = allocate(intArrayFromString(string), 'i8', ALLOC_NORMAL);
+        GLEW.error[error] = allocateUTF8(string);
       }
       return GLEW.error[error];
     },
@@ -92,30 +93,30 @@ var LibraryGLEW = {
         var string = GLEW.versionStringConstantFromCode(name);
         if (!string)
           return 0;
-        GLEW.version[name] = allocate(intArrayFromString(string), 'i8', ALLOC_NORMAL);
+        GLEW.version[name] = allocateUTF8(string);
       }
       return GLEW.version[name];
     },
 
     extensionIsSupported: function(name) {
       if (!GLEW.extensions) {
-        GLEW.extensions = Pointer_stringify(_glGetString(0x1F03)).split(' ');
+        GLEW.extensions = UTF8ToString(_glGetString(0x1F03)).split(' ');
       }
 
-      if (GLEW.extensions.indexOf(name) != -1)
+      if (GLEW.extensions.includes(name))
         return 1;
 
       // extensions from GLEmulations do not come unprefixed
       // so, try with prefix
-      return (GLEW.extensions.indexOf("GL_" + name) != -1);
+      return (GLEW.extensions.includes("GL_" + name));
     },
   },
 
   glewInit: function() { return 0; },
 
   glewIsSupported: function(name) {
-    var exts = Pointer_stringify(name).split(' ');
-    for (var i in exts) {
+    var exts = UTF8ToString(name).split(' ');
+    for (var i = 0; i < exts.length; ++i) {
       if (!GLEW.extensionIsSupported(exts[i]))
         return 0;
     }
@@ -123,9 +124,10 @@ var LibraryGLEW = {
   },
 
   glewGetExtension: function(name) {
-    return GLEW.extensionIsSupported(Pointer_stringify(name));
+    return GLEW.extensionIsSupported(UTF8ToString(name));
   },
 
+  glewGetErrorString__sig: 'ii',
   glewGetErrorString: function(error) {
     return GLEW.errorString(error);
   },
