@@ -1,30 +1,30 @@
 #include <assert.h>
 #include <emscripten/console.h>
 #include <setjmp.h>
+#include <malloc.h>
+#include <stdlib.h>
 
 jmp_buf env;
 
 void luaWork(int d){
   int x;
-  _emscripten_outf("d is at %d\n", d);
+  _emscripten_outf("d is at %d", d);
+  assert(d <= 10);
 
   longjmp(env, 1);
 }
 
-#include <malloc.h>
-#include <stdlib.h>
-
 void dump() {
   struct mallinfo m = mallinfo();
-  _emscripten_outf("dump: %d , %d\n", m.arena, m.uordblks);
+  _emscripten_outf("dump: %d , %d", m.arena, m.uordblks);
 }
 
-void work(int n)
-{
-  _emscripten_outf("work %d\n", n);
+void work(int n) {
+  _emscripten_outf("work %d", n);
   dump();
 
-  if(!setjmp(env)){
+  _emscripten_outf("calling luaWork with n=%d", n);
+  if (!setjmp(env)){
     luaWork(n);
   }
 
@@ -38,5 +38,5 @@ int main() {
   dump();
   struct mallinfo m2 = mallinfo();
   assert(m1.uordblks == m2.uordblks);
-  _emscripten_out("ok.\n");
+  _emscripten_out("ok.");
 }
