@@ -12,10 +12,12 @@ import shutil
 import textwrap
 from enum import IntEnum, auto
 from glob import iglob
+from typing import List, Optional
 
 from . import shared, building, utils
 from . import deps_info
 from . import diagnostics
+from . import cache
 from tools.shared import demangle_c_symbol_name
 from tools.settings import settings
 
@@ -285,7 +287,7 @@ class Library:
   # automatically get the correct version of the library.
   # This should only be overridden in a concrete library class, e.g. libc,
   # and left as None in an abstract library class, e.g. MTLibrary.
-  name = None
+  name: Optional[str] = None
 
   # Set to true to prevent EMCC_FORCE_STDLIBS from linking this library.
   never_force = False
@@ -304,7 +306,7 @@ class Library:
   # directory into the include path, you would write:
   #    includes = [('system', 'lib', 'a'), ('system', 'lib', 'b')]
   # The include path of the parent class is automatically inherited.
-  includes = []
+  includes: List[str] = []
 
   # By default, `get_files` look for source files for this library under `src_dir`.
   # It will either use the files listed in `src_files`, or use the glob pattern in
@@ -312,10 +314,10 @@ class Library:
   # When using `src_glob`, you can specify a list of files in `src_glob_exclude`
   # to be excluded from the library.
   # Alternatively, you can override `get_files` to use your own logic.
-  src_dir = None
-  src_files = None
-  src_glob = None
-  src_glob_exclude = None
+  src_dir: Optional[str] = None
+  src_files: Optional[List[str]] = []
+  src_glob: Optional[str] = None
+  src_glob_exclude: Optional[List[str]] = None
 
   # Whether to always generate WASM object files, even when LTO is set
   force_object_files = False
@@ -357,10 +359,10 @@ class Library:
     return True
 
   def erase(self):
-    shared.Cache.erase_file(self.get_path())
+    cache.erase_file(self.get_path())
 
   def get_path(self, absolute=False):
-    return shared.Cache.get_lib_name(self.get_filename(), absolute=absolute)
+    return cache.get_lib_name(self.get_filename(), absolute=absolute)
 
   def build(self, deterministic_paths=False):
     """
@@ -369,7 +371,7 @@ class Library:
     This will trigger a build if this library is not in the cache.
     """
     self.deterministic_paths = deterministic_paths
-    return shared.Cache.get(self.get_path(), self.do_build, force=USE_NINJA == 2, quiet=USE_NINJA)
+    return cache.get(self.get_path(), self.do_build, force=USE_NINJA == 2, quiet=USE_NINJA)
 
   def get_link_flag(self):
     """
@@ -478,7 +480,7 @@ class Library:
   def do_build(self, out_filename):
     """Builds the library and returns the path to the file."""
     assert out_filename == self.get_path(absolute=True)
-    build_dir = os.path.join(shared.Cache.get_path('build'), self.get_base_name())
+    build_dir = os.path.join(cache.get_path('build'), self.get_base_name())
     if USE_NINJA:
       self.build_with_ninja(build_dir, out_filename)
     else:
@@ -2192,26 +2194,26 @@ def install_system_headers(stamp):
     ('lib', 'libcxxabi', 'include'): os.path.join('c++', 'v1'),
   }
 
-  target_include_dir = shared.Cache.get_include_dir()
+  target_include_dir = cache.get_include_dir()
   for src, dest in install_dirs.items():
     src = utils.path_from_root('system', *src)
     dest = os.path.join(target_include_dir, dest)
     copytree_exist_ok(src, dest)
 
   pkgconfig_src = utils.path_from_root('system/lib/pkgconfig')
-  pkgconfig_dest = shared.Cache.get_sysroot_dir('lib/pkgconfig')
+  pkgconfig_dest = cache.get_sysroot_dir('lib/pkgconfig')
   copytree_exist_ok(pkgconfig_src, pkgconfig_dest)
 
   bin_src = utils.path_from_root('system/bin')
-  bin_dest = shared.Cache.get_sysroot_dir('bin')
+  bin_dest = cache.get_sysroot_dir('bin')
   copytree_exist_ok(bin_src, bin_dest)
 
   cmake_src = utils.path_from_root('system/lib/cmake')
-  cmake_dest = shared.Cache.get_sysroot_dir('lib/cmake')
+  cmake_dest = cache.get_sysroot_dir('lib/cmake')
   copytree_exist_ok(cmake_src, cmake_dest)
 
   # Create a version header based on the emscripten-version.txt
-  version_file = shared.Cache.get_include_dir('emscripten/version.h')
+  version_file = cache.get_include_dir('emscripten/version.h')
   utils.write_file(version_file, textwrap.dedent(f'''\
   /* Automatically generated by tools/system_libs.py */
   #define __EMSCRIPTEN_major__ {shared.EMSCRIPTEN_VERSION_MAJOR}
@@ -2229,4 +2231,4 @@ def install_system_headers(stamp):
 
 @ToolchainProfiler.profile()
 def ensure_sysroot():
-  shared.Cache.get('sysroot_install.stamp', install_system_headers, what='system headers')
+  cache.get('sysroot_install.stamp', install_system_headers, what='system headers')
